/************************************************************
 *This program sets up and test the modes of timer 0 by     *
 *prompting the user to enter which mode that they desire   *
 *timer 0 to be in, then timer 0 is set to that mode and    *
 *the timer 0 interrupt is enabled.  When an interrupt is   *
 *generated the interrupt service routine is executed.  The *
 *LEDs are illuminated to show that the program has executed*
 *correctly.												*
 ************************************************************/
#include<Reg930.h>
#include<stdio.h>
#define DELAY 10000
//#pragma iv(0x4000)

extern void serialSetup(void);
void mainStart(void);

/*Pseudo main function is used to allow space for interrupt vectors.*/
void main(void)
{
	/*True main function is called with address starting at 0x4040.*/
	mainStart();
}

void mainStart(void)
{
	float mode;

  	/*Serial communication is set up with this function and the TI bit
      must be set in the interrupt register.*/
	serialSetup();	

	/*Print out the user menu and get the user's input.*/
	puts("\n\nThe following timer modes may be selected for timer0\n\n");
	puts("Mode 0 - 13 bit timer\n");
	puts("Mode 1 - 16 bit timer\n");
	puts("Mode 2 - 8 bit timer with Auto-reload\n");
 	puts("Mode 3 - two 8 bit timers\n");  
	printf("Enter the Mode number that you would like timer 0 to be set as and press \nenter: ");
	scanf("%f", &mode);
	printf("\nThe mode is %.2f\n", mode);
	TI = 0;
		
	/*The timers are turned off in the Timer control register*/
	TCON = 0x00; 

	
	/*The mode of timer 0 is set in the timer mode register.*/
    /*Mode 0 -> 13 bit timers*/
   	if (mode == 0)  
  	{
  		TMOD = 0x00; 
  		goto setbits;
  	 }

     /*Mode 1 -> 16 bit timers*/
 	 else if (mode == 1)
    {
		TMOD = 0x01; 
     	goto setbits;
    }

	/*Mode 2 -> 8 bit timer with Auto-reload*/
    else if (mode == 2)
    {
		TMOD = 0x02; 
       	goto setbits;
    }
    
    /*Mode 3 -> two 8 bit timers*/
	else if (mode == 3)
    {
		TMOD = 0x03; 
        goto setbits;
    }
	
	/*Incorrect mode is selected.*/
	else
	{
		puts("\n\nYou have entered the wrong value, reset and start again.");
		while(1);
	}

    /*The interrupt for timer 0 is enabled and timer 0 is turned on.*/
	setbits:
		ET0 = 1; 
		TR0 = 1; 
	while(1);

}

/*Interrupt service routine for timer 0, interrupt vector located
  at 0x000B but actually 0x400B*/
void timer0Isr(void) interrupt 1 using 1
{
	short cnt; 

	/*The interrupt is disabled.*/
	ET1 = 0;  

	/*LEDs are turned on to show that interrupt service routine was exectuted*/
       	for (cnt = DELAY; cnt > 0; cnt--)   
	   		P1 = 0x40;
   		for (cnt = DELAY; cnt > 0; cnt--)
			P1 = 0x20;
		for (cnt = DELAY; cnt > 0; cnt--)
			P1 = 0x10;
		for (cnt = DELAY; cnt > 0; cnt--)
			P1 = 0x08;
}





